# -*- coding: utf-8 -*-

from django import forms
from django.forms.models import inlineformset_factory
from django.test import TestCase

from django_stachoutils.forms import ModelForm
from .models import Car, CarOption


CarOptionFormSet = inlineformset_factory(Car, CarOption, fields=['name'])


class CarForm(ModelForm):
    pk = forms.IntegerField(widget=forms.HiddenInput, required=False)

    class Meta:
        model = Car
        fields = ('name', 'brand')

    class Forms:
        inlines = {
            'options': CarOptionFormSet
        }


class ModelFormTest(TestCase):
    def test_render_model_form(self):
        form = CarForm()
        self.assertHTMLEqual(
            form.as_table(),
            """<tr>
                <th><label for="id_name">Name:</label></th>
                <td><input type="text" name="name" required id="id_name" maxlength="20" /></td>
               </tr>
               <tr>
                 <th><label for="id_brand">Brand:</label></th>
                 <td>
                   <input type="text" name="brand" required id="id_brand" maxlength="100" />
                   <input id="id_pk" name="pk" type="hidden" />
                 </td>
               </tr>"""
        )
        self.assertHTMLEqual(
            form.as_tr(),
            """<td>
                 <input type="text" name="name" required id="id_name" maxlength="20" />
               </td>
               <td>
                 <input type="text" name="brand" required id="id_brand" maxlength="100" />
               </td>
               <input id="id_pk" name="pk" type="hidden" /><span class="row_end" />"""
        )
        self.assertHTMLEqual(
            form.labels_as_tr(),
            """<tr><th><label for="id_name">Name</label></th><th><label for="id_brand">Brand</label></th></tr>"""
        )
        self.assertHTMLEqual(
            form.inlineformsets['options'].management_form.as_table(),
            """<input type="hidden" name="OPTIONS-TOTAL_FORMS" value="3" id="id_OPTIONS-TOTAL_FORMS" />
            <input type="hidden" name="OPTIONS-INITIAL_FORMS" value="0" id="id_OPTIONS-INITIAL_FORMS" />
            <input type="hidden" name="OPTIONS-MIN_NUM_FORMS" value="0" id="id_OPTIONS-MIN_NUM_FORMS" />
            <input type="hidden" name="OPTIONS-MAX_NUM_FORMS" value="1000" id="id_OPTIONS-MAX_NUM_FORMS" />"""
        )
        self.assertHTMLEqual(
            form.inlineformsets['options'].as_table(),
            """<input type="hidden" name="OPTIONS-TOTAL_FORMS" value="3" id="id_OPTIONS-TOTAL_FORMS" />
               <input type="hidden" name="OPTIONS-INITIAL_FORMS" value="0" id="id_OPTIONS-INITIAL_FORMS" />
               <input type="hidden" name="OPTIONS-MIN_NUM_FORMS" value="0" id="id_OPTIONS-MIN_NUM_FORMS" />
               <input type="hidden" name="OPTIONS-MAX_NUM_FORMS" value="1000" id="id_OPTIONS-MAX_NUM_FORMS" />
               <tr><th><label for="id_OPTIONS-0-name">Name:</label></th>
                   <td><input type="text" name="OPTIONS-0-name" id="id_OPTIONS-0-name" maxlength="15" /></td>
               </tr>
               <tr><th><label for="id_OPTIONS-0-DELETE">Delete:</label></th>
                   <td><input type="checkbox" name="OPTIONS-0-DELETE" id="id_OPTIONS-0-DELETE" />
                       <input type="hidden" name="OPTIONS-0-id" id="id_OPTIONS-0-id" />
                       <input type="hidden" name="OPTIONS-0-car" id="id_OPTIONS-0-car" />
                   </td>
               </tr>
               <tr>
                 <th><label for="id_OPTIONS-1-name">Name:</label></th>
                 <td><input type="text" name="OPTIONS-1-name" id="id_OPTIONS-1-name" maxlength="15" /></td>
              </tr>
              <tr>
                <th><label for="id_OPTIONS-1-DELETE">Delete:</label></th>
                <td><input type="checkbox" name="OPTIONS-1-DELETE" id="id_OPTIONS-1-DELETE" />
                    <input type="hidden" name="OPTIONS-1-id" id="id_OPTIONS-1-id" />
                    <input type="hidden" name="OPTIONS-1-car" id="id_OPTIONS-1-car" />
                </td>
              </tr>
              <tr>
                <th><label for="id_OPTIONS-2-name">Name:</label></th>
                <td><input type="text" name="OPTIONS-2-name" id="id_OPTIONS-2-name" maxlength="15" /></td>
              </tr>
              <tr>
                <th><label for="id_OPTIONS-2-DELETE">Delete:</label></th>
                <td><input type="checkbox" name="OPTIONS-2-DELETE" id="id_OPTIONS-2-DELETE" />
                    <input type="hidden" name="OPTIONS-2-id" id="id_OPTIONS-2-id" />
                    <input type="hidden" name="OPTIONS-2-car" id="id_OPTIONS-2-car" />
                </td>
              </tr>
            """
        )

    def test_model_form_not_valid(self):
        form = CarForm({
            'name': '9.3 2.0t',
            'OPTIONS-INITIAL_FORMS': '0',
            'OPTIONS-TOTAL_FORMS': '3',
            'OPTIONS-MIN_NUM_FORMS': '0',
            'OPTIONS-MAX_NUM_FORMS': '1000',
        })
        self.assertFalse(form.is_valid())

    def test_model_form_is_valid(self):
        form = CarForm({
            'name': '9.3 2.0t',
            'brand': 'Saab',
            'OPTIONS-INITIAL_FORMS': '0',
            'OPTIONS-TOTAL_FORMS': '3',
            'OPTIONS-MIN_NUM_FORMS': '0',
            'OPTIONS-MAX_NUM_FORMS': '1000',
        })
        self.assertTrue(form.is_valid())

    def test_model_form_with_inlines_not_valid(self):
        form = CarForm({
            'name': '900 Turbo 16',
            'brand': 'Saab',
            'OPTIONS-INITIAL_FORMS': '0',
            'OPTIONS-TOTAL_FORMS': '3',
            'OPTIONS-MIN_NUM_FORMS': '0',
            'OPTIONS-MAX_NUM_FORMS': '1000',
            'OPTIONS-0-name': 'Climatisation',
            'OPTIONS-1-name': 'Cruise-Control',
            'OPTIONS-2-name': 'Super extra audio sound system',
        })
        self.assertFalse(form.is_valid())

    def test_model_form_with_inlines_is_valid(self):
        form = CarForm({
            'name': '900 Turbo 16',
            'brand': 'Saab',
            'OPTIONS-INITIAL_FORMS': '0',
            'OPTIONS-TOTAL_FORMS': '3',
            'OPTIONS-MIN_NUM_FORMS': '0',
            'OPTIONS-MAX_NUM_FORMS': '1000',
            'OPTIONS-0-name': 'Climatisation',
            'OPTIONS-1-name': 'Cruise-Control',
            'OPTIONS-2-name': 'Sound system',
        })
        self.assertTrue(form.is_valid())

    def test_model_form_save(self):
        form = CarForm({
            'name': '9.3 2.0t',
            'brand': 'Saab',
            'OPTIONS-INITIAL_FORMS': '0',
            'OPTIONS-TOTAL_FORMS': '3',
            'OPTIONS-MIN_NUM_FORMS': '0',
            'OPTIONS-MAX_NUM_FORMS': '1000',
        })
        form.save()
        my_car = Car.objects.latest('pk')
        self.assertEqual(my_car.brand, 'Saab')
        self.assertEqual(my_car.name, '9.3 2.0t')
        self.assertEqual(my_car.caroption_set.all().count(), 0)

        form = CarForm({
            'id': my_car.pk,
            'name': '900 Turbo 16',
            'brand': 'Saab',
            'OPTIONS-INITIAL_FORMS': '0',
            'OPTIONS-TOTAL_FORMS': '3',
            'OPTIONS-MIN_NUM_FORMS': '0',
            'OPTIONS-MAX_NUM_FORMS': '1000',
        })
        self.assertTrue(form.has_changed())

    def test_model_form_save_with_inlines(self):
        form = CarForm({
            'name': '900 Turbo 16',
            'brand': 'Saab',
            'OPTIONS-INITIAL_FORMS': '0',
            'OPTIONS-TOTAL_FORMS': '3',
            'OPTIONS-MIN_NUM_FORMS': '0',
            'OPTIONS-MAX_NUM_FORMS': '1000',
            'OPTIONS-0-name': 'Climatisation',
            'OPTIONS-1-name': 'Cruise-Control',
            'OPTIONS-2-name': 'Alarm',
        })
        form.save()
        my_car = Car.objects.latest('pk')
        self.assertEqual(my_car.brand, 'Saab')
        self.assertEqual(my_car.name, '900 Turbo 16')
        self.assertEqual(my_car.caroption_set.all().count(), 3)

        form = CarForm({
            'id': my_car.pk,
            'name': '900 Turbo 16',
            'brand': 'Saab',
            'OPTIONS-INITIAL_FORMS': '3',
            'OPTIONS-TOTAL_FORMS': '3',
            'OPTIONS-MIN_NUM_FORMS': '0',
            'OPTIONS-MAX_NUM_FORMS': '1000',
            'OPTIONS-0-id': 1,
            'OPTIONS-0-name': 'Climatisation',
            'OPTIONS-1-id': 2,
            'OPTIONS-1-name': 'Cruise-Control',
            'OPTIONS-2-id': 3,
            'OPTIONS-2-name': 'Alarm',
        }, instance=my_car)
        self.assertFalse(form.has_changed())

        form = CarForm({
            'id': my_car.pk,
            'name': '900 Turbo 16',
            'brand': 'Saab',
            'OPTIONS-INITIAL_FORMS': '3',
            'OPTIONS-TOTAL_FORMS': '3',
            'OPTIONS-MIN_NUM_FORMS': '0',
            'OPTIONS-MAX_NUM_FORMS': '1000',
            'OPTIONS-0-id': 1,
            'OPTIONS-0-name': 'Air-conditioning',
            'OPTIONS-1-id': 2,
            'OPTIONS-1-name': 'Cruise-Control',
            'OPTIONS-2-id': 3,
            'OPTIONS-2-name': 'Alarm',
        }, instance=my_car)
        self.assertTrue(form.has_changed())


class PrefixedModelFormTest(TestCase):
    def test_render_model_form(self):
        form = CarForm(prefix="customer")
        self.maxDiff = None
        self.assertHTMLEqual(
            form.as_table(),
            """<tr>
                <th><label for="id_customer-name">Name:</label></th>
                <td><input type="text" name="customer-name" required id="id_customer-name" maxlength="20" /></td>
               </tr>
               <tr>
                 <th><label for="id_customer-brand">Brand:</label></th>
                 <td>
                   <input type="text" name="customer-brand" required id="id_customer-brand" maxlength="100" />
                   <input id="id_customer-pk" name="customer-pk" type="hidden" />
                 </td>
               </tr>"""
        )
        self.assertHTMLEqual(
            form.as_tr(),
            """<td>
                 <input type="text" name="customer-name" required id="id_customer-name" maxlength="20" />
               </td>
               <td>
                 <input type="text" name="customer-brand" required id="id_customer-brand" maxlength="100" />
               </td>
               <input id="id_customer-pk" name="customer-pk" type="hidden" /><span class="row_end" />"""
        )
        self.assertHTMLEqual(
            form.labels_as_tr(),
            """<tr><th><label for="id_customer-name">Name</label></th><th><label for="id_customer-brand">Brand</label></th></tr>"""
        )
        self.assertHTMLEqual(
            form.inlineformsets['options'].management_form.as_table(),
            """<input type="hidden" name="customer_OPTIONS-TOTAL_FORMS" value="3" id="id_customer_OPTIONS-TOTAL_FORMS" />
            <input type="hidden" name="customer_OPTIONS-INITIAL_FORMS" value="0" id="id_customer_OPTIONS-INITIAL_FORMS" />
            <input type="hidden" name="customer_OPTIONS-MIN_NUM_FORMS" value="0" id="id_customer_OPTIONS-MIN_NUM_FORMS" />
            <input type="hidden" name="customer_OPTIONS-MAX_NUM_FORMS" value="1000" id="id_customer_OPTIONS-MAX_NUM_FORMS" />"""
        )
        self.assertHTMLEqual(
            form.inlineformsets['options'].as_table(),
            """<input type="hidden" name="customer_OPTIONS-TOTAL_FORMS" value="3" id="id_customer_OPTIONS-TOTAL_FORMS" />
               <input type="hidden" name="customer_OPTIONS-INITIAL_FORMS" value="0" id="id_customer_OPTIONS-INITIAL_FORMS" />
               <input type="hidden" name="customer_OPTIONS-MIN_NUM_FORMS" value="0" id="id_customer_OPTIONS-MIN_NUM_FORMS" />
               <input type="hidden" name="customer_OPTIONS-MAX_NUM_FORMS" value="1000" id="id_customer_OPTIONS-MAX_NUM_FORMS" />
               <tr><th><label for="id_customer_OPTIONS-0-name">Name:</label></th>
                   <td><input type="text" name="customer_OPTIONS-0-name" id="id_customer_OPTIONS-0-name" maxlength="15" /></td>
               </tr>
               <tr><th><label for="id_customer_OPTIONS-0-DELETE">Delete:</label></th>
                   <td><input type="checkbox" name="customer_OPTIONS-0-DELETE" id="id_customer_OPTIONS-0-DELETE" />
                       <input type="hidden" name="customer_OPTIONS-0-id" id="id_customer_OPTIONS-0-id" />
                       <input type="hidden" name="customer_OPTIONS-0-car" id="id_customer_OPTIONS-0-car" />
                   </td>
               </tr>
               <tr>
                 <th><label for="id_customer_OPTIONS-1-name">Name:</label></th>
                 <td><input type="text" name="customer_OPTIONS-1-name" id="id_customer_OPTIONS-1-name" maxlength="15" /></td>
              </tr>
              <tr>
                <th><label for="id_customer_OPTIONS-1-DELETE">Delete:</label></th>
                <td><input type="checkbox" name="customer_OPTIONS-1-DELETE" id="id_customer_OPTIONS-1-DELETE" />
                    <input type="hidden" name="customer_OPTIONS-1-id" id="id_customer_OPTIONS-1-id" />
                    <input type="hidden" name="customer_OPTIONS-1-car" id="id_customer_OPTIONS-1-car" />
                </td>
              </tr>
              <tr>
                <th><label for="id_customer_OPTIONS-2-name">Name:</label></th>
                <td><input type="text" name="customer_OPTIONS-2-name" id="id_customer_OPTIONS-2-name" maxlength="15" /></td>
              </tr>
              <tr>
                <th><label for="id_customer_OPTIONS-2-DELETE">Delete:</label></th>
                <td><input type="checkbox" name="customer_OPTIONS-2-DELETE" id="id_customer_OPTIONS-2-DELETE" />
                    <input type="hidden" name="customer_OPTIONS-2-id" id="id_customer_OPTIONS-2-id" />
                    <input type="hidden" name="customer_OPTIONS-2-car" id="id_customer_OPTIONS-2-car" />
                </td>
              </tr>
            """
        )

    def test_model_form_not_valid(self):
        form = CarForm({
            'customer-name': '9.3 2.0t',
            'customer_OPTIONS-INITIAL_FORMS': '0',
            'customer_OPTIONS-TOTAL_FORMS': '3',
            'customer_OPTIONS-MIN_NUM_FORMS': '0',
            'customer_OPTIONS-MAX_NUM_FORMS': '1000',
        }, prefix="customer")
        self.assertFalse(form.is_valid())

    def test_model_form_is_valid(self):
        form = CarForm({
            'customer-name': '9.3 2.0t',
            'customer-brand': 'Saab',
            'customer_OPTIONS-INITIAL_FORMS': '0',
            'customer_OPTIONS-TOTAL_FORMS': '3',
            'customer_OPTIONS-MIN_NUM_FORMS': '0',
            'customer_OPTIONS-MAX_NUM_FORMS': '1000',
        }, prefix="customer")
        self.assertTrue(form.is_valid())

    def test_model_form_with_inlines_not_valid(self):
        form = CarForm({
            'customer-name': '900 Turbo 16',
            'customer-brand': 'Saab',
            'customer_OPTIONS-INITIAL_FORMS': '0',
            'customer_OPTIONS-TOTAL_FORMS': '3',
            'customer_OPTIONS-MIN_NUM_FORMS': '0',
            'customer_OPTIONS-MAX_NUM_FORMS': '1000',
            'customer_OPTIONS-0-name': 'Climatisation',
            'customer_OPTIONS-1-name': 'Cruise-Control',
            'customer_OPTIONS-2-name': 'Super extra audio sound system',
        }, prefix="customer")
        self.assertFalse(form.is_valid())

    def test_model_form_with_inlines_is_valid(self):
        form = CarForm({
            'customer-name': '900 Turbo 16',
            'customer-brand': 'Saab',
            'customer_OPTIONS-INITIAL_FORMS': '0',
            'customer_OPTIONS-TOTAL_FORMS': '3',
            'customer_OPTIONS-MIN_NUM_FORMS': '0',
            'customer_OPTIONS-MAX_NUM_FORMS': '1000',
            'customer_OPTIONS-0-name': 'Climatisation',
            'customer_OPTIONS-1-name': 'Cruise-Control',
            'customer_OPTIONS-2-name': 'Sound system',
        }, prefix="customer")
        self.assertTrue(form.is_valid())

    def test_model_form_save(self):
        form = CarForm({
            'customer-name': '9.3 2.0t',
            'customer-brand': 'Saab',
            'customer_OPTIONS-INITIAL_FORMS': '0',
            'customer_OPTIONS-TOTAL_FORMS': '3',
            'customer_OPTIONS-MIN_NUM_FORMS': '0',
            'customer_OPTIONS-MAX_NUM_FORMS': '1000',
        }, prefix="customer")
        form.save()
        my_car = Car.objects.latest('pk')
        self.assertEqual(my_car.brand, 'Saab')
        self.assertEqual(my_car.name, '9.3 2.0t')
        self.assertEqual(my_car.caroption_set.all().count(), 0)

        form = CarForm({
            'customer-id': my_car.pk,
            'customer-name': '900 Turbo 16',
            'customer-brand': 'Saab',
            'customer_OPTIONS-INITIAL_FORMS': '0',
            'customer_OPTIONS-TOTAL_FORMS': '3',
            'customer_OPTIONS-MIN_NUM_FORMS': '0',
            'customer_OPTIONS-MAX_NUM_FORMS': '1000',
        }, prefix="customer")
        self.assertTrue(form.has_changed())

    def test_model_form_save_with_inlines(self):
        form = CarForm({
            'customer-name': '900 Turbo 16',
            'customer-brand': 'Saab',
            'customer_OPTIONS-INITIAL_FORMS': '0',
            'customer_OPTIONS-TOTAL_FORMS': '3',
            'customer_OPTIONS-MIN_NUM_FORMS': '0',
            'customer_OPTIONS-MAX_NUM_FORMS': '1000',
            'customer_OPTIONS-0-name': 'Climatisation',
            'customer_OPTIONS-1-name': 'Cruise-Control',
            'customer_OPTIONS-2-name': 'Alarm',
        }, prefix="customer")
        form.save()
        my_car = Car.objects.latest('pk')
        self.assertEqual(my_car.brand, 'Saab')
        self.assertEqual(my_car.name, '900 Turbo 16')
        self.assertEqual(my_car.caroption_set.all().count(), 3)

        form = CarForm({
            'customer-id': my_car.pk,
            'customer-name': '900 Turbo 16',
            'customer-brand': 'Saab',
            'customer_OPTIONS-INITIAL_FORMS': '3',
            'customer_OPTIONS-TOTAL_FORMS': '3',
            'customer_OPTIONS-MIN_NUM_FORMS': '0',
            'customer_OPTIONS-MAX_NUM_FORMS': '1000',
            'customer_OPTIONS-0-id': 1,
            'customer_OPTIONS-0-name': 'Climatisation',
            'customer_OPTIONS-1-id': 2,
            'customer_OPTIONS-1-name': 'Cruise-Control',
            'customer_OPTIONS-2-id': 3,
            'customer_OPTIONS-2-name': 'Alarm',
        }, prefix="customer", instance=my_car)
        self.assertFalse(form.has_changed())

        form = CarForm({
            'customer-id': my_car.pk,
            'customer-name': '900 Turbo 16',
            'customer-brand': 'Saab',
            'customer_OPTIONS-INITIAL_FORMS': '3',
            'customer_OPTIONS-TOTAL_FORMS': '3',
            'customer_OPTIONS-MIN_NUM_FORMS': '0',
            'customer_OPTIONS-MAX_NUM_FORMS': '1000',
            'customer_OPTIONS-0-id': 1,
            'customer_OPTIONS-0-name': 'Air-conditioning',
            'customer_OPTIONS-1-id': 2,
            'customer_OPTIONS-1-name': 'Cruise-Control',
            'customer_OPTIONS-2-id': 3,
            'customer_OPTIONS-2-name': 'Alarm',
        }, prefix="customer" , instance=my_car)
        self.assertTrue(form.has_changed())



from django.db import models


class Car(models.Model):
    name = models.CharField(max_length=20)
    brand = models.CharField(max_length=100)
    description = models.TextField(null=True, blank=True)
    owner = models.ForeignKey('Person', on_delete=models.CASCADE, null=True)

    def __str__(self):
        return self.name


class CarOption(models.Model):
    car = models.ForeignKey('Car', on_delete=models.CASCADE)
    name = models.CharField(max_length=15)


class Person(models.Model):
    name = models.CharField(max_length=100)
    lastname = models.CharField(max_length=100)
    gender = models.CharField(choices=[('male', 'Male'), ('female', 'Female')], max_length=6)
    house = models.ForeignKey('House', on_delete=models.CASCADE, null=True)


def house_photo_path(instance, filename):
    return 'houses/{}_{}.jpg'.format(instance.address, instance.city)


class House(models.Model):
    address = models.CharField(max_length=100)
    city = models.CharField(max_length=100)
    photo = models.ImageField(upload_to=house_photo_path, null=True)



import os

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

SECRET_KEY = 'i0_+-t@@wul&q)30+4y)8-19s)31@%cv8$q(c@8q1g#h$6wn-='

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
    }
}

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.contenttypes',
]

MEDIA_URL = 'http://media.my_project.com/'
MEDIA_ROOT = os.path.join(CURRENT_DIR, 'media')

THUMBOR_SERVER = 'http://my_thumbor.com:8888'
THUMBOR_SERVER_EXTERNAL = THUMBOR_SERVER


ADMINS=[('Some dude', 'admin@myproject.com')]
EMAIL_SUBJECT_PREFIX='[Project Lambda]'
EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend'
EMAIL_HOST = 'localhost'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'simple': {
            'format': '%(levelname)s %(asctime)s %(message)s',
        },
    },
    'handlers': {
        'null': {
            'level': 'ERROR',
            'class': 'django_stachoutils.log.NullHandler',
        },
        'message': {
            'level': 'WARNING',
            'class': 'django_stachoutils.log.MessagelHandler',
            'formatter': 'simple'
        },
        'mail_admins_buffered': {
            'level': 'INFO',
            'class': 'django_stachoutils.log.BufferingAdminEmailHandler',
            'subject': "[MY PROJECT]",
        },
        'mail_admins': {
            'level': 'INFO',
            'class': 'django_stachoutils.log.AdminEmailHandler',
            'subject': "[MY PROJECT]",
        },
    },
    'loggers': {
        'django.DisallowedHost': {
            'handlers': ['null'],
            'propagate': True,
        },
        'stachoutils.logger1': {
            'handlers': ['message'],
            'level': 'INFO',
            'propagate': True,
        },
        'stachoutils.logger2': {
            'handlers': ['mail_admins'],
            'level': 'WARNING',
            'propagate': True,
        },
        'stachoutils.logger3': {
            'handlers': ['mail_admins_buffered'],
            'level': 'WARNING',
            'propagate': True,
        },
        'stachoutils.logger4': {
            'handlers': ['null'],
            'level': 'INFO',
            'propagate': True,
        },
    }
}



from future import standard_library
standard_library.install_aliases()
from builtins import str
import csv
import decimal
import datetime
import zipfile

from io import BytesIO
from django.http import HttpResponse
from django.shortcuts import _get_queryset
from django_stachoutils import csv_utf8


def macroman_text_response(data, filename=None, encode=True):
    if encode:
        data = data.encode('macroman')
    response = HttpResponse(data, content_type="text/plain; charset=macintosh")
    if filename:
        response['Content-Disposition'] = 'attachment; filename="%s"' % filename
    return response


def encode_default(d):
    if isinstance(d, decimal.Decimal):
        return float(str(d))
    elif isinstance(d, datetime.date) or isinstance(d, datetime.datetime):
        return d.isoformat()
    raise TypeError


def get_object_or_none(klass, *args, **kwargs):
    queryset = _get_queryset(klass)
    try:
        return queryset.get(*args, **kwargs)
    except queryset.model.DoesNotExist:
        return None


def xml_response(data, filename=None):
    response = HttpResponse(data, content_type='application/xml')
    if filename:
        response['Content-Disposition'] = 'attachment; filename="%s"' % filename
    return response


def zip_response(files, filename):
    response = HttpResponse(content_type='application/zip')
    response['Content-Disposition'] = 'filename=%s' % filename

    buffer = BytesIO()
    zip = zipfile.ZipFile(buffer, "w", zipfile.ZIP_DEFLATED)
    for name, bytes in files:
        zip.writestr(name, bytes)
    zip.close()
    buffer.flush()
    ret_zip = buffer.getvalue()
    buffer.close()
    response.write(ret_zip)
    return response


def csv_response(rows, filename, kwargs={'delimiter': ',', 'quoting': csv.QUOTE_ALL}):
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="%s"' % filename

    writer = csv_utf8.UnicodeWriter(response, **kwargs)
    writer.writerows(rows)
    return response


def createAndAppendElement(document, parentNode, tagName, text=None):
    node = document.createElement(tagName)
    parentNode.appendChild(node)
    if text:
        node.appendChild(document.createTextNode(text))
    return node



# -*- coding: utf-8 -*-

from django.test import TestCase


class TemplateTagsTest(TestCase):
    def test_current_filters_html(self):
        from django_stachoutils.templatetags.stachoutils_extras import current_filters
        filters = [
            (u'statut', [
                ('valide__exact=1&assistant__exact=6', 'All', True),
                ('valide__exact=1&assistant__exact=6&statut__exact=1', u'Relation Client', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=2', u'Production', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=3', u'P.A.O.', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=4', u'Publicité à Valider', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=5', u'Accord Bon à Tirer', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=6', u'Publicité Modifiée', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=7', u'OK Fab', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=8', u'Correction Demandées', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=9', u'BAT Bloqué', False)
            ]),
            (u'BAT demandé', [
                ('valide__exact=1&assistant__exact=6', 'All', None),
                ('valide__exact=1&assistant__exact=6&bat_demande__exact=1', 'Oui', False),
                ('valide__exact=1&assistant__exact=6&bat_demande__exact=0', 'Non', True)
            ]),
            (u'chargé de clientèle', [
                ('valide__exact=1', 'All', None),
                ('valide__exact=1&assistant__exact=8', u'Isabelle G.', False),
                ('valide__exact=1&assistant__exact=6', u'Marie-Françoise B.', True),
                ('valide__exact=1&assistant__exact=41', u'Émilie G.', False),
                ('valide__exact=1&assistant__exact=1445', u'Séverine T.', False),
                ('assistant__isnull=True&valide__exact=1&assistant__exact=6', u'Aucun', None)
            ]),
        ]

        self.assertEqual(current_filters(filters),
                         [u'BAT demandé: <strong>Non</strong>',
                          u'chargé de clientèle: <strong>Marie-Françoise B.</strong>'])

        # Should works with a filter not selected.
        filters = [
            (u'statut', [
                ('valide__exact=1&assistant__exact=6', 'All', None),
                ('valide__exact=1&assistant__exact=6&statut__exact=1', u'Relation Client', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=2', u'Production', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=3', u'P.A.O.', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=4', u'Publicité à Valider', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=5', u'Accord Bon à Tirer', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=6', u'Publicité Modifiée', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=7', u'OK Fab', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=8', u'Correction Demandées', False),
                ('valide__exact=1&assistant__exact=6&statut__exact=9', u'BAT Bloqué', False)
            ]),
            (u'BAT demandé', [
                ('valide__exact=1&assistant__exact=6', 'All', None),
                ('valide__exact=1&assistant__exact=6&bat_demande__exact=1', 'Oui', False),
                ('valide__exact=1&assistant__exact=6&bat_demande__exact=0', 'Non', True)
            ]),
            (u'chargé de clientèle', [
                ('valide__exact=1', 'All', None),
                ('valide__exact=1&assistant__exact=8', u'Isabelle G.', False),
                ('valide__exact=1&assistant__exact=6', u'Marie-Françoise B.', True),
                ('valide__exact=1&assistant__exact=41', u'Émilie G.', False),
                ('valide__exact=1&assistant__exact=1445', u'Séverine T.', False),
                ('assistant__isnull=True&valide__exact=1&assistant__exact=6', u'Aucun', None)
            ]),
        ]

        self.assertEqual(current_filters(filters),
                         [u'BAT demandé: <strong>Non</strong>',
                          u'chargé de clientèle: <strong>Marie-Françoise B.</strong>'])



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from collections import OrderedDict
from django.apps import apps
from django.core.management.base import BaseCommand, CommandError
from django.core import serializers
from django.db import router, DEFAULT_DB_ALIAS


class Command(BaseCommand):
    help = ("Output the contents of the database as a fixture of the given "
            "format (using each model's default manager unless --all is "
            "specified).")

    def add_arguments(self, parser):
        parser.add_argument('app_labels', nargs="*", help='[app_label app_label.ModelName ...]')
        parser.add_argument('--format', default="json", help='Specifies the output serialization format for fixtures.')
        parser.add_argument('--indent', type=int, default=None, help='Specifies the indent level to use when pretty-printing output')
        parser.add_argument('--database', default=DEFAULT_DB_ALIAS, help='Nominates a specific database to dump fixtures from. Defaults to the "default" database.')
        parser.add_argument('--exclude', action='append', default=[], help='An appname or appname.ModelName to exclude (use multiple --exclude to exclude multiple apps/models).')
        parser.add_argument('--natural-foreign', action='store_true', dest='use_natural_foreign_keys', default=False, help='Use natural foreign keys if they are available.')
        parser.add_argument('--natural-primary', action='store_true', dest='use_natural_primary_keys', default=False, help='Use natural primary keys if they are available.')
        parser.add_argument('-a', '--all', action='store_true', dest='use_base_manager', default=False, help="Use Django's base manager to dump all models stored in the database, including those that would otherwise be filtered or modified by a custom manager.")
        parser.add_argument('--pks', dest='primary_keys', help="Only dump objects with given primary keys. Accepts a comma separated list of keys. This option will only work when you specify one model.")
        parser.add_argument('-o', '--output', default=None, help='Specifies file to which the output is written.')
        parser.add_argument('--related')

    def handle(self, *args, **options):
        app_labels = options.get('app_labels')
        format = options.get('format')
        indent = options.get('indent')
        using = options.get('database')
        excludes = options.get('exclude')
        output = options.get('output')
        show_traceback = options.get('traceback')
        use_natural_foreign_keys = options.get('use_natural_foreign_keys')
        use_natural_primary_keys = options.get('use_natural_primary_keys')
        use_base_manager = options.get('use_base_manager')
        pks = options.get('primary_keys')
        related = options.get('related')

        if pks:
            primary_keys = pks.split(',')
        else:
            primary_keys = []

        if related:
            related = related.split(',')

        excluded_apps = set()
        excluded_models = set()
        for exclude in excludes:
            if '.' in exclude:
                try:
                    model = apps.get_model(exclude)
                except LookupError:
                    raise CommandError('Unknown model in excludes: %s' % exclude)
                excluded_models.add(model)
            else:
                try:
                    app_config = apps.get_app_config(exclude)
                except LookupError:
                    raise CommandError('Unknown app in excludes: %s' % exclude)
                excluded_apps.add(app_config)

        if len(app_labels) == 0:
            if primary_keys:
                raise CommandError("You can only use --pks option with one model")
            app_list = OrderedDict(
                (app_config, None) for app_config in apps.get_app_configs()
                if app_config.models_module is not None and app_config not in excluded_apps
            )
        else:
            if len(app_labels) > 1 and primary_keys:
                raise CommandError("You can only use --pks option with one model")
            app_list = OrderedDict()
            for label in app_labels:
                try:
                    app_label, model_label = label.split('.')
                    try:
                        app_config = apps.get_app_config(app_label)
                    except LookupError:
                        raise CommandError("Unknown application: %s" % app_label)
                    if app_config.models_module is None or app_config in excluded_apps:
                        continue
                    try:
                        model = app_config.get_model(model_label)
                    except LookupError:
                        raise CommandError("Unknown model: %s.%s" % (app_label, model_label))

                    app_list_value = app_list.setdefault(app_config, [])

                    # We may have previously seen a "all-models" request for
                    # this app (no model qualifier was given). In this case
                    # there is no need adding specific models to the list.
                    if app_list_value is not None:
                        if model not in app_list_value:
                            app_list_value.append(model)
                except ValueError:
                    if primary_keys:
                        raise CommandError("You can only use --pks option with one model")
                    # This is just an app - no model qualifier
                    app_label = label
                    try:
                        app_config = apps.get_app_config(app_label)
                    except LookupError:
                        raise CommandError("Unknown application: %s" % app_label)
                    if app_config.models_module is None or app_config in excluded_apps:
                        continue
                    app_list[app_config] = None

        if related:
            if len(app_list) != 1 or len(list(app_list.values())[0]) != 1:
                raise CommandError("you can only use --related option with one model")

        # Check that the serialization format exists; this is a shortcut to
        # avoid collating all the objects and _then_ failing.
        if format not in serializers.get_public_serializer_formats():
            try:
                serializers.get_serializer(format)
            except serializers.SerializerDoesNotExist:
                pass

            raise CommandError("Unknown serialization format: %s" % format)

        def get_objects():
            # Collate the objects to be serialized.
            models = serializers.sort_dependencies(app_list.items())
            for model in models:
                if model in excluded_models:
                    continue
                if not model._meta.proxy and router.allow_migrate(using, model):
                    if use_base_manager:
                        objects = model._base_manager
                    else:
                        objects = model._default_manager

                    queryset = objects.using(using).order_by(model._meta.pk.name)
                    if primary_keys:
                        queryset = queryset.filter(pk__in=primary_keys)
                        if related:
                            queryset = queryset.select_related()
                    serialized_objects = set()
                    for obj in queryset.iterator():
                        if related:
                            for rel_expression in related:
                                for rel_obj in get_related_objects(obj, rel_expression):
                                    if rel_obj not in serialized_objects:
                                        serialized_objects.add(rel_obj)
                                        yield rel_obj
                        if obj not in serialized_objects:
                            yield obj

        def get_related_objects(obj, rel_expression):
            expressions = rel_expression.split('.')
            nb_expressions = len(expressions)
            rel_objs = [obj]
            for i, fieldname in enumerate(expressions):
                last_iteration = (i == (nb_expressions - 1)) and True or False
                rel_objs = [getattr(rel_obj, fieldname) for rel_obj in rel_objs
                            if getattr(rel_obj, fieldname)]
                # FK is null, don't go any further.
                if not rel_objs:
                    return rel_objs
                # M2M.
                if hasattr(rel_objs[0], "all"):
                    rel_objs = [rel for rel_obj in rel_objs for rel in rel_obj.all()]
                    if use_natural_foreign_keys and last_iteration and \
                       hasattr(obj, 'natural_key') and len(rel_objs):
                        # Search a potential FK to main object. If such
                        # relation exists, obj is inserted to respect the
                        # dependency order.
                        rel_obj = rel_objs[0]
                        for field in rel_obj._meta.fields:
                            if field.remote_field and field.remote_field.model == obj.__class__ and \
                               getattr(rel_obj, field.name) == obj:
                                rel_objs.insert(0, obj)
                                break

                # Retrieve OneToOne relations if such exists.
                if len(rel_objs) and last_iteration:
                    parents = rel_objs[0]._meta.get_parent_list()
                    if parents:
                        parents = [parent.objects.get(pk=rel.pk)
                                   for rel in rel_objs for parent in parents]
                        rel_objs = parents + rel_objs
            return rel_objs

        try:
            self.stdout.ending = None
            serializers.serialize(format, get_objects(), indent=indent,
                                  use_natural_foreign_keys=use_natural_foreign_keys,
                                  use_natural_primary_keys=use_natural_primary_keys,
                                  stream=open(output, 'w') if output else self.stdout)
        except Exception as e:
            if show_traceback:
                raise
            raise CommandError("Unable to serialize database: %s" % e)



# -*- coding: utf-8 -*-

import os
import shutil
import tempfile

from django import forms
from django.conf import settings
from django.core.files.images import ImageFile
from django.test import TestCase

from django_stachoutils.forms import ImageModelChoiceField

from .models import House, Person


class PersonForm(forms.ModelForm):
    house = ImageModelChoiceField(related_fieldname="photo",
                                  queryset=House.objects.all())

    class Meta:
        model = Person
        fields = ('name', 'lastname', 'gender', 'house')


class FieldsTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.h1 = House.objects.create(address="12, rue de la Paix", city="Paris")
        cls.h2 = House.objects.create(address="4, rue du Cherche-Midi", city="Paris")

    def setUp(self):
        self.tmp_media_root = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmp_media_root)

    def test_render_image_model_choice_field(self):
        form = PersonForm()
        self.assertHTMLEqual(
            form.as_table(),
            """
            <tr>
              <th><label for="id_name">Name:</label></th>
              <td><input type="text" name="name" required id="id_name" maxlength="100" /></td>
            </tr>
            <tr>
              <th><label for="id_lastname">Lastname:</label></th>
              <td><input type="text" name="lastname" required id="id_lastname" maxlength="100" /></td>
            </tr>
            <tr>
              <th><label for="id_gender">Gender:</label></th>
              <td>
                <select name="gender" required id="id_gender">
                  <option value="" selected>---------</option>
                  <option value="male">Male</option>
                  <option value="female">Female</option>
                </select>
                <div class="droppableHiddenInput">
                  <input type="hidden" name="house" />
                  <div class="droppableContainer">
                    <span class="delete" title="Vider l'emplacement"></span>
                    <div class="droppable">
                      <div class="draggable"><img /></div>
                    </div>
                  </div>
                  <div class="message"></div>
                </div>
              </td>
            </tr>
            """
        )

    def test_bound_image_model_choice_field(self):
        form = PersonForm({'name': 'Stan',
                           'lastname': 'Guerra',
                           'gender': 'male'})
        self.assertFalse(form.is_valid())

        old_media_root = settings.MEDIA_ROOT
        with self.settings(MEDIA_ROOT=self.tmp_media_root):
            with open(os.path.join(old_media_root, "1st_Saab_9-3_SE.jpg"), mode="rb") as f:
                self.h1.photo.save("some pic", ImageFile(f))
            form = PersonForm({'name': 'Stan',
                               'lastname': 'Guerra',
                               'gender': 'male',
                               'house': self.h1.pk})
            self.assertHTMLEqual(
                form.as_table(),
                """
                <tr>
                  <th>
                    <label for="id_name">Name:</label>
                  </th>
                  <td>
                    <input type="text" name="name" value="Stan" required id="id_name" maxlength="100" />
                  </td>
                </tr>
                <tr>
                  <th><label for="id_lastname">Lastname:</label></th>
                  <td><input type="text" name="lastname" value="Guerra" required id="id_lastname" maxlength="100" /></td>
                </tr>
                <tr>
                  <th><label for="id_gender">Gender:</label></th>
                  <td>
                    <select name="gender" required id="id_gender">
                      <option value="">---------</option>
                      <option value="male" selected>Male</option>
                      <option value="female">Female</option>
                    </select>
                    <div class="droppableHiddenInput">
                      <input type="hidden" name="house" value="1" />
                      <div class="droppableContainer">
                        <span class="delete" title="Vider l'emplacement"></span>
                        <div class="droppable">
                          <div class="draggable">
                            <img src="{}cache/5f/7f/5f7fd7419603eb53e5519c1eda28e81b.jpg"width="120" height="58">
                          </div>
                        </div>
                      </div>
                      <div class="message"></div>
                    </div>
                  </td>
                </tr>
                """.format(settings.MEDIA_URL)
            )
            self.assertTrue(form.is_valid())
            form.save()

            person = Person.objects.latest('pk')
            self.assertEqual(person.name, 'Stan')
            self.assertEqual(person.house, self.h1)



# -*- coding: utf-8 -*-

from .fields import ImageModelChoiceField
from .models import ModelFormOptions, ModelFormMetaclass, ModelForm
from .nested import NestedModelFormOptions, NestedModelFormMetaclass, NestedModelForm, NESTED_NON_FIELD_ERRORS
from .widgets import ImageDroppableHiddenInput, get_thumbor_thumbnail_url, get_thumbor_thumbnail_tag



import logging

from django.contrib.messages.storage.base import BaseStorage
from django.core import mail
from django.test import RequestFactory, TestCase


class MessageStorage(BaseStorage):
    """This is a basic messages storage since RequestFactory does not provide
       a complete request object according to settings.MIDDLEWARE.
    """
    def __init__(self, request, *args, **kwargs):
        self.messages = None
        super(MessageStorage, self).__init__(request, *args, **kwargs)

    def _get(self, *args, **kwargs):
        return self.messages, True

    def _store(self, messages, response, *args, **kwargs):
        if messages:
            self.messages = messages
        else:
            self.messages = None
        return []


class MessageHandlerTest(TestCase):
    def test_message_handler(self):
        logger = logging.getLogger("stachoutils.logger1")
        rf = RequestFactory()
        request = rf.get('/')
        request._messages = MessageStorage(request)

        logger.error("This is a error message", extra={'request': request})
        logger.info("This is a info message", extra={'request': request})  # ignored
        self.assertEqual([str(m) for m in request._messages], ["This is a error message"])


# Can't use @override_settings here since BufferingSMTPHandler is initialized
# with settings values.
class AdminEmailHandlerTest(TestCase):
    def test_admin_email_handler(self):
        logger = logging.getLogger("stachoutils.logger2")
        logger.error("This is a error message")
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, '[Project Lambda][ERROR] : [MY PROJECT]')
        self.assertEqual(mail.outbox[0].body, 'This is a error message')

    def test_buffered_admin_email_handler(self):
        logger = logging.getLogger("stachoutils.logger3")
        for i in range(3):
            logger.error("This is error message #%d" % i)
        self.assertEqual(len(mail.outbox), 0)

        logger.handlers[0].flush()
        self.assertEqual(len(mail.outbox), 1)


class NullHandlerTest(TestCase):
    def test_null_handler(self):
        logger = logging.getLogger("stachoutils.logger4")
        logger.error("This error is not handled")



# -*- coding: utf-8 -*-

from django import forms
from django.test import TestCase

from .models import Car


class WidgetsTest(TestCase):
    def test_render_admin_texte_editeur_widget(self):
        from django_stachoutils.forms.widgets import AdminTexteEditeur

        class CarForm(forms.ModelForm):
            description = forms.CharField(widget=AdminTexteEditeur,
                                          label="Describe the car",
                                          required=False)

            class Meta:
                model = Car
                fields = ('name', 'brand', 'description')

        form = CarForm()
        self.assertHTMLEqual(
            form.as_table(),
            """
            <tr>
              <th><label for="id_name">Name:</label></th>
              <td><input type="text" name="name" required id="id_name" maxlength="20" /></td>
            </tr>
            <tr>
              <th><label for="id_brand">Brand:</label></th>
              <td><input type="text" name="brand" required id="id_brand" maxlength="100" /></td>
            </tr>
            <tr>
              <th><label for="id_description">Describe the car:</label></th>
              <td>
                <div class="editeur">
                  <div class="toolbar">
                    <button type="button" accesskey="g" class="gras">Gras</button>
                    <button type="button" accesskey="s" class="sup">Supérieur</button>
                    <button type="button" accesskey="b" class="bas">Bas de casse</button>
                    <button type="button" accesskey="G" class="gras_phrases">Gras phrases</button>
                  </div>
                  <textarea name="description" id="id_description" rows="10" cols="40" class="vLargeTextField">
                  </textarea>
                </div>
              </td>
            </tr>
            """
        )
        self.assertHTMLEqual(
            str(form.media),
            """
            <script type="text/javascript" src="/static/django_stachoutils/js/jquery.charcounter.js"></script>
            <script type="text/javascript" src="/static/django_stachoutils/js/jquery.fieldselection.js"></script>
            <script type="text/javascript" src="/static/django_stachoutils/js/editeur.js"></script>
            """
        )

    def test_render_textarea_counter_widget(self):
        from django_stachoutils.forms.widgets import TextareaCounter

        class CarForm(forms.ModelForm):
            description = forms.CharField(widget=TextareaCounter(max_signs=200),
                                          label="Describe the car",
                                          required=False)

            class Meta:
                model = Car
                fields = ('name', 'brand', 'description')

        form = CarForm()
        self.assertHTMLEqual(
            form.as_table(),
            """
            <tr>
              <th><label for="id_name">Name:</label></th>
              <td><input type="text" name="name" required id="id_name" maxlength="20" /></td>
            </tr>
            <tr>
              <th><label for="id_brand">Brand:</label></th>
              <td><input type="text" name="brand" required id="id_brand" maxlength="100" /></td>
            </tr>
            <tr>
              <th><label for="id_description">Describe the car:</label></th>
              <td>
                <div class="editeur">
                  <div class="toolbar">
                    <button type="button" accesskey="g" class="gras">Gras</button>
                    <button type="button" accesskey="s" class="sup">Supérieur</button>
                    <button type="button" accesskey="b" class="bas">Bas de casse</button>
                    <button type="button" accesskey="G" class="gras_phrases">Gras phrases</button>
                  </div>
                  <textarea name="description" id="id_description" rows="10" cols="40" class="vLargeTextField">
                  </textarea>
                </div>
                <script type="text/javascript">$(document).ready(function () { $("#id_description").charCounter(200, {container: "<div></div>",direction: "up",format: "%1 signe(s)",limit: false,limit_class: "error"});});</script>
              </td>
            </tr>
            """
        )
        self.assertHTMLEqual(
            str(form.media),
            """
            <script type="text/javascript" src="/static/django_stachoutils/js/jquery.charcounter.js"></script>
            <script type="text/javascript" src="/static/django_stachoutils/js/jquery.fieldselection.js"></script>
            <script type="text/javascript" src="/static/django_stachoutils/js/editeur.js"></script>
            """
        )

    def test_render_textarea_counter_widget_reversed_counter(self):
        from django_stachoutils.forms.widgets import TextareaCounter

        class CarForm(forms.ModelForm):
            description = forms.CharField(widget=TextareaCounter(max_signs=200, direction='down'),
                                          label="Describe the car",
                                          required=False)

            class Meta:
                model = Car
                fields = ('name', 'brand', 'description')

        form = CarForm()
        self.assertHTMLEqual(
            form.as_table(),
            """
            <tr>
              <th><label for="id_name">Name:</label></th>
              <td><input type="text" name="name" required id="id_name" maxlength="20" /></td>
            </tr>
            <tr>
              <th><label for="id_brand">Brand:</label></th>
              <td><input type="text" name="brand" required id="id_brand" maxlength="100" /></td>
            </tr>
            <tr>
              <th><label for="id_description">Describe the car:</label></th>
              <td>
                <div class="editeur">
                  <div class="toolbar">
                    <button type="button" accesskey="g" class="gras">Gras</button>
                    <button type="button" accesskey="s" class="sup">Supérieur</button>
                    <button type="button" accesskey="b" class="bas">Bas de casse</button>
                    <button type="button" accesskey="G" class="gras_phrases">Gras phrases</button>
                  </div>
                  <textarea name="description" id="id_description" rows="10" cols="40" class="vLargeTextField">
                  </textarea>
                </div>
                <script type="text/javascript">$(document).ready(function () { $("#id_description").charCounter(200, {container: "<div></div>",direction: "down",format: "%1 signe(s) restant(s)",limit: false,limit_class: "error"});});</script>
              </td>
            </tr>
            """
        )

    def test_render_textinput_counter_widget(self):
        from django_stachoutils.forms.widgets import TextInputCounter

        class CarForm(forms.ModelForm):
            name = forms.CharField(widget=TextInputCounter(max_signs=100),
                                          label="Name of the car",
                                          required=True)

            class Meta:
                model = Car
                fields = ('name', 'brand')

        form = CarForm()
        self.assertHTMLEqual(
            form.as_table(),
            """
            <tr>
              <th><label for="id_name">Name of the car:</label></th>
              <td>
                <input type="text" name="name" required id="id_name" />
                <script type="text/javascript">$(document).ready(function () { $("#id_name").charCounter(100, {container: "<span></span>",direction: "up",format: "%1 signe(s)",limit: false,limit_class: "error"});});</script>
              </td>
            </tr>
            <tr>
              <th><label for="id_brand">Brand:</label></th>
              <td><input type="text" name="brand" required id="id_brand" maxlength="100" /></td>
            </tr>
            """
        )
        self.assertHTMLEqual(
            str(form.media),
            """
            <script type="text/javascript" src="/static/django_stachoutils/js/jquery.charcounter.js"></script>
            <script type="text/javascript" src="/static/django_stachoutils/js/jquery.fieldselection.js"></script>
            <script type="text/javascript" src="/static/django_stachoutils/js/editeur.js"></script>
            """
        )

    def test_render_textinput_counter_widget_reversed_counter(self):
        from django_stachoutils.forms.widgets import TextInputCounter

        class CarForm(forms.ModelForm):
            name = forms.CharField(widget=TextInputCounter(max_signs=100, direction='down'),
                                          label="Name of the car",
                                          required=True)

            class Meta:
                model = Car
                fields = ('name', 'brand')

        form = CarForm()
        self.assertHTMLEqual(
            form.as_table(),
            """
            <tr>
              <th><label for="id_name">Name of the car:</label></th>
              <td>
                <input type="text" name="name" required id="id_name" />
                <script type="text/javascript">$(document).ready(function () { $("#id_name").charCounter(100, {container: "<span></span>",direction: "down",format: "%1 signe(s) restant(s)",limit: false,limit_class: "error"});});</script>
              </td>
            </tr>
            <tr>
              <th><label for="id_brand">Brand:</label></th>
              <td><input type="text" name="brand" required id="id_brand" maxlength="100" /></td>
            </tr>
            """
        )



# -*- coding: utf-8 -*-

from builtins import object
from django import forms
from django.forms.models import ModelFormMetaclass
from django.shortcuts import get_object_or_404
from future.utils import with_metaclass

NESTED_NON_FIELD_ERRORS = '__nested__'


class NestedModelFormOptions(object):
    def __init__(self, options=None):
        self.Form = getattr(options, 'form', None)  # Class.
        self.fk = getattr(options, 'fk', None)      # fk name.


class NestedModelFormMetaclass(ModelFormMetaclass):
    """ Ajoute les attributs self._nested.Form et self._nested.fk"""
    def __new__(cls, name, bases, attrs):
        new_class = super(NestedModelFormMetaclass, cls).__new__(cls, name, bases, attrs)
        # Declarez une class Nested.
        new_class._nested = NestedModelFormOptions(getattr(new_class, 'Nested', None))
        return new_class


class NestedModelForm(with_metaclass(NestedModelFormMetaclass, forms.ModelForm)):
    def __init__(self, *args, **kwargs):
        self.forms = []
        self.nested_form = None
        self._non_nested_errors = None
        nested_initial, nested_instance = {}, None
        if ('initial' in kwargs) and ('NESTED' in kwargs['initial']):
            nested_initial = kwargs['initial'].pop('NESTED')
        super(NestedModelForm, self).__init__(*args, **kwargs)
        # self.initial est un merge du paramètre 'initial' et des attrs de l'instance.
        nested_id = self.initial.get(self._nested.fk, None)
        # POST écrase initial si présent.
        if '%s-%s' % (self.prefix, self._nested.fk) in self.data:
            nested_id = self.data['%s-%s' % (self.prefix, self._nested.fk)]
        if nested_id:
            nested_instance = get_object_or_404(self._nested.Form.Meta.model, pk=nested_id)
        self._init_nested_form(nested_instance, nested_initial)

    def _init_nested_form(self, instance, initial):
        prefix = '%s_%s' % (self.prefix, self._nested.fk.upper())
        self.nested_form = self._nested.Form(self.data or None, self.files or None,
                                             prefix=prefix, initial=initial, instance=instance)
        self.forms.append(self.nested_form)

    def _get_errors(self):
        if self._errors is None:
            super(NestedModelForm, self).errors
            if not self.is_bound:  # Stop further processing.
                return self._errors
            if self.has_changed() or self.nested_form.has_changed():
                self.nested_form.empty_permitted = True
            nested_errors = self.nested_form.errors
            if nested_errors:
                self._errors[NESTED_NON_FIELD_ERRORS] = self.nested_form.non_field_errors()
        return self._errors
    errors = property(_get_errors)

    def _get_non_nested_errors(self):
        if self._non_nested_errors is None:
            self._non_nested_errors = self.errors.copy()
            if NESTED_NON_FIELD_ERRORS in self._non_nested_errors:
                del self._non_nested_errors[NESTED_NON_FIELD_ERRORS]
        return self._non_nested_errors
    non_nested_errors = property(_get_non_nested_errors)

    def has_changed(self, *args, **kwargs):
        nested_has_changed = self.nested_form.has_changed()
        host_has_changed = super(NestedModelForm, self).has_changed(*args, **kwargs)
        if host_has_changed or nested_has_changed:
            return True
        self.nested_form._errors = {}  # Why do we need to do that ? If not, next
        # call to errors (i.e. from the template) will revalidate and add errors although
        # It (the nested form) has not changed.
        return False

    def save(self, *args, **kwargs):
        instance = self.instance
        self.nested_instance = None
        if self.nested_form.has_changed():
            self.nested_instance = self.nested_form.save()
        instance = super(NestedModelForm, self).save(*args, **kwargs)
        if self.nested_instance:
            setattr(instance, self._nested.fk, self.nested_instance)
            instance.save()
        return instance



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from builtins import chr, map, range, str
import hashlib
import re
import unicodedata

from datetime import datetime
from django.utils import formats, dateformat
from django.utils.encoding import force_str
from django.utils.functional import keep_lazy_text
from django.utils.http import urlunquote

from .decorators import simple_decorator  # NOQA


def format_datetime(value=None, date=True, time=True):
    if not date:
        format_type = 'TIME_FORMAT'
    elif not time:
        format_type = 'DATE_FORMAT'
    else:
        format_type = 'DATETIME_FORMAT'
    return dateformat.format(value or datetime.now(), formats.get_format(format_type))


def format_number(value=None, separator=','):
    if not value:
        return ''

    def split_thousands(s):
        if len(s) <= 3:
            return s
        return split_thousands(s[:-3]) + separator + s[-3:]
    return split_thousands(str(value))


# From http://stackoverflow.com/questions/517923/what-is-the-best-way-to-remove-accents-in-a-python-unicode-string
def strip_accents(txt):
    return unicodedata.normalize('NFD', str(txt)).encode('ascii', 'ignore').decode('ascii')


def latin1_safe_xml_encode(txt):
    return str(txt.encode('latin-1', 'xmlcharrefreplace'), 'latin-1')


xml_entity_table = bytes.maketrans(b' \'', b'--')


def format_xml_entity(txt):
    return strip_accents(txt).encode('ascii').upper().translate(xml_entity_table)


@keep_lazy_text
def truncate_chars(s, num):
    "Truncates a string after a certain number of characters."
    txt = force_str(s)
    length = int(num)
    if len(txt) > length:
        txt = txt[:length] + '...'
    return txt


rx_lettrine = re.compile(r"(\w)(\s)([\w-]+)([,.]{0,1}\s.*)", re.UNICODE | re.IGNORECASE)


def clean_lettrine(txt):
    """Vire les espaces ajoutés aux textes suite import QuarkXpress"""
    match = rx_lettrine.match(txt)
    if match:
        match = match.groups()
        supp_premier_espace = True
        if match[0] == 'A':
            if match[2] not in ('u', 'ux', 'vec', 'utour',):
                supp_premier_espace = False
        if supp_premier_espace:
            txt = rx_lettrine.sub(r'\1\3\4', txt)
    return txt


# From http://www.daniweb.com/code/snippet216865.html
#
# convert an integer to a roman numeral
# keep it reasonable since from 5000 on special characters are used
# see also: http://en.wikipedia.org/wiki/Roman_numerals
# tested with Python24       vegaseat        25jan2007
def int_to_roman(number):
    numerals = {1: "I", 4: "IV", 5: "V", 9: "IX", 10: "X", 40: "XL",
                50: "L", 90: "XC", 100: "C", 400: "CD", 500: "D", 900: "CM", 1000: "M"}
    result = ""
    for value, numeral in sorted(list(numerals.items()), reverse=True):
        while number >= value:
            result += numeral
            number -= value
    return result


# Inspired by http://stackoverflow.com/questions/92438/stripping-non-printable-characters-from-a-string-in-python
non_printable_re = re.compile('[%s]' % re.escape(''.join(map(chr, list(range(0, 9)) + list(range(11, 13)) + list(range(14, 32)) + list(range(127, 160))))))


def filter_non_printable(txt):
    return non_printable_re.sub('', txt)


def camelize(txt):
    return ' '.join([t.title() for t in txt.split(' ')])


def files_are_equal(f1, f2):
    if f1.size != f2.size:
        return False
    elif _digest_file(f1) != _digest_file(f2):
        return False
    return True


def _digest_file(f, chunk_size=8192):
    md5 = hashlib.md5()
    for chunk in iter(lambda: f.read(chunk_size), b""):
        md5.update(chunk)
    f.seek(0)
    return md5.digest()


def urldecode(url):
    out = {}
    get_params = (url.split('?')[1:] or [None])
    if get_params[0]:
        for param in get_params[0].split('&'):
            k, v = param.split('=')
            out[k] = urlunquote(v)
    return out



from django.core.paginator import EmptyPage
from django.test import RequestFactory, TestCase

from django_stachoutils import options
from .models import Car


class PaginateTest(TestCase):
    def test_paginate(self):
        Car.objects.create(brand='Saab', name='9.3')
        Car.objects.create(brand='Saab', name='900')
        Car.objects.create(brand='Saab', name='9.5')
        Car.objects.create(brand='Alfa-Romeo', name='Giullia')
        Car.objects.create(brand='Alfa-Romeo', name='Sprint')
        Car.objects.create(brand='Subaru', name='Forester')
        Car.objects.create(brand='Subaru', name='Impreza')

        queryset = Car.objects.all().order_by('brand', 'name')
        rf = RequestFactory()

        request = rf.get('/')
        page = options.paginate(request, queryset, paginate_by=4)
        self.assertEqual(len(page.object_list), 4)
        self.assertEqual(page.next_page_number(), 2)

        request = rf.get('/?page=2')
        page = options.paginate(request, queryset, paginate_by=4)
        self.assertEqual(len(page.object_list), 3)
        self.assertRaises(EmptyPage, page.next_page_number)

        request = rf.get('/?page=2&paginate_by=2')
        page = options.paginate(request, queryset)
        self.assertEqual(len(page.object_list), 2)
        self.assertEqual(page.next_page_number(), 3)

        request = rf.get('/?page=2&paginate_by=All')
        page = options.paginate(request, queryset)
        self.assertEqual(len(page.object_list), 7)
        self.assertRaises(EmptyPage, page.next_page_number)

        # With an invalid 'page' parameter, paginate will returns the first one.
        request = rf.get('/?page=two')
        page = options.paginate(request, queryset, paginate_by=4)
        self.assertEqual(page.number, 1)

        # With an 'page' parameter greater than last page, paginate will returns the last one.
        request = rf.get('/?page=5')
        page = options.paginate(request, queryset, paginate_by=4)
        self.assertEqual(page.number, 2)

        # With an invalid 'paginate_by' GET parameter, paginate will use the default value.
        request = rf.get('/?page=2&paginate_by=five')
        page = options.paginate(request, queryset)
        self.assertFalse(page.has_next())



from django.db import models


class InstrumentManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)


class MusicGenreManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)


class BandManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)


class BandMemberManager(models.Manager):
    def get_by_natural_key(self, first_name, last_name):
        return self.get(first_name=first_name, last_name=last_name)


def can_search_instrument(instance, user):
    return user and user.is_authenticated() or False

class Instrument(models.Model):
    objects = InstrumentManager()
    name = models.CharField(max_length=64, unique=True)

    class Yaaac:
        user_passes_test = can_search_instrument
        allows_suggest_by = ['__unicode__']

    def __unicode__(self):
        return self.name

    def natural_key(self):
        return (self.name,)

    def get_absolute_url(self):
        return "http://en.wikipedia.org/wiki/%s" % self.name


class MusicGenre(models.Model):
    objects = MusicGenreManager()
    name = models.CharField(max_length=64, unique=True)
    history = models.TextField(null=True, blank=True)

    def __unicode__(self):
        return self.name

    def natural_key(self):
        return (self.name,)


class Band(models.Model):
    objects = BandManager()
    name = models.CharField(max_length=100, unique=True)
    genre = models.ForeignKey("MusicGenre", null=True, blank=True)

    class Yaaac:
        user_passes_test = lambda instance, u: True
        allows_suggest_by = ['name', 'get_full_info']

    def __unicode__(self):
        return self.name

    def natural_key(self):
        return (self.name,)

    def get_full_info(self):
        return u"%s (%s)" % (self.name, self.genre)


class BandMember(models.Model):
    objects = BandMemberManager()
    first_name = models.CharField(max_length=100)
    last_name = models.CharField(max_length=100)
    band = models.ForeignKey("Band", null=True, blank=True)
    favorite_instrument = models.ForeignKey("Instrument", null=True, blank=True)
    story = models.TextField(null=True, blank=True)

    class Meta:
        unique_together = (('first_name', 'last_name'),)

    class Yaaac:
        user_passes_test = lambda instance, u: True
        allows_suggest_by = ['get_full_name']

    def __unicode__(self):
        return u"%s %s" % (self.first_name, self.last_name)

    def natural_key(self):
        return (self.first_name, self.last_name)

    def get_full_name(self):
        return u"%s %s" % (self.first_name, self.last_name)



from django.db import models


class Car(models.Model):
    name = models.CharField(max_length=100)
    brand = models.CharField(max_length=100)
    description = models.TextField(null=True, blank=True)

    def __str__(self):
        return self.name



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst'), encoding='utf-8') as f:
    README = f.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name="django-stachoutils",
    version="3.2.4",
    license='BSD Licence',
    author='Stanislas Guerra',
    author_email='stan@slashdev.me',
    description='Commons for Django',
    url='https://github.com/Starou/django-stachoutils',
    project_urls={
        'Source Code': 'https://github.com/Starou/django-stachoutils',
        'Issue Tracker': 'https://github.com/Starou/django-stachoutils/issues',
    },
    long_description=README,
    install_requires=['future'],
    packages=[
        'django_stachoutils',
        'django_stachoutils.management',
        'django_stachoutils.management.commands',
        'django_stachoutils.forms',
        'django_stachoutils.views',
        'django_stachoutils.templatetags',
    ],
    package_data={
        'django_stachoutils': [
            'static/django_stachoutils/css/*.css',
            'static/django_stachoutils/js/*.js',
            'static/django_stachoutils/img/*.png',
            'static/django_stachoutils/img/*.jpg',
            'static/django_stachoutils/img/*.gif',
            'static/django_stachoutils/poshytip/*.js',
            'static/django_stachoutils/poshytip/*/*.css',
            'static/django_stachoutils/poshytip/*/*.gif',
            'static/django_stachoutils/poshytip/*/*.png',
            'templates/django_stachoutils/*.html',
            'templates/django_stachoutils/forms/widgets/*.html',
        ]
    },
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ]
)



from builtins import str
from builtins import object


# https://gist.github.com/sidmitra/646372
class ExceptionUserInfoMiddleware(object):
    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        return self.get_response(request)

    def process_exception(self, request, exception):
        try:
            if request.user.is_authenticated:
                request.META['USERNAME'] = str(request.user.username)
                request.META['USER_EMAIL'] = str(request.user.email)
        except:
            pass



# -*- coding: utf-8 -*-

from builtins import str
from django import forms
from django import VERSION as DJ_VERSION
from django.contrib.admin.models import LogEntry
from django.contrib.auth.models import User
from django.forms.models import inlineformset_factory
from django.test import RequestFactory, TestCase

from django_stachoutils import options
from .models import Car, CarOption


class CarForm(forms.ModelForm):
    class Meta:
        model = Car
        fields = ('name', 'brand')


class LogsTest(TestCase):
    rf = RequestFactory()
    @classmethod
    def setUpTestData(cls):
        cls.user = User.objects.create_user('john', 'lennon@thebeatles.com', 'johnpassword')

    def setUp(self):
        self.request = self.rf.get('/')
        self.request.user = self.user

    def test_log_addition(self):
        car = Car.objects.create(brand='Saab', name='9-3')
        options.log_addition(self.request, car)

        log = LogEntry.objects.latest('pk')
        self.assertTrue(log.is_addition())
        self.assertEqual(log.object_repr, u'9-3')
        self.assertEqual(log.object_id, str(car.pk))
        self.assertEqual(log.user_id, self.user.pk)

    def test_log_change(self):
        car = Car.objects.create(brand='Saab', name='9-3')
        car.name = '9-3t Biopower'
        options.log_change(self.request, car, "Fixed the name of the damned car")

        log = LogEntry.objects.latest('pk')
        self.assertTrue(log.is_change())
        self.assertEqual(log.object_repr, u'9-3t Biopower')
        self.assertEqual(log.object_id, str(car.pk))
        self.assertEqual(log.user_id, self.user.pk)

    def test_log_deletion(self):
        car = Car.objects.create(brand='Saab', name='9-3')
        options.log_deletion(self.request, car, str(car))

        log = LogEntry.objects.latest('pk')
        self.assertTrue(log.is_deletion())
        self.assertEqual(log.object_repr, u'9-3')
        self.assertEqual(log.object_id, str(car.pk))
        self.assertEqual(log.user_id, self.user.pk)

    def test_log_change_for_user(self):
        car = Car.objects.create(brand='Saab', name='9-3')
        car.name = '9-3t Biopower'
        options.log_change_for_user(self.user, car, "Fixed the name of the damned car")

        log = LogEntry.objects.latest('pk')
        self.assertTrue(log.is_change())
        self.assertEqual(log.object_repr, u'9-3t Biopower')
        self.assertEqual(log.object_id, str(car.pk))
        self.assertEqual(log.user_id, self.user.pk)

    def test_logs_for_forms(self):
        # Addition
        form = CarForm({
            'name': '9.3 2.0t',
            'brand': 'Saab',
        })
        car = form.save()
        options.logs_for_formsets('addition', self.request, car, [form])
        log = LogEntry.objects.latest('pk')
        self.assertTrue(log.is_addition())
        self.assertEqual(log.user, self.user)
        if DJ_VERSION >= (3, 0):
            self.assertEqual(str(log), 'Added “9.3 2.0t”.')
        else:
            self.assertEqual(str(log), 'Added "9.3 2.0t".')

        # Modification
        form = CarForm({
            'id': car.pk,
            'name': '900 Turbo 16',
            'brand': 'Saab',
        }, instance=car)
        car = form.save()
        options.logs_for_formsets('change', self.request, car, [form])
        log = LogEntry.objects.latest('pk')
        self.assertFalse(log.is_addition())
        self.assertEqual(log.user, self.user)
        if DJ_VERSION >= (3, 0):
            self.assertEqual(str(log), u'Changed “900 Turbo 16” — a modifié : name (car [900 Turbo 16]) ; ')
        else:
            self.assertEqual(str(log), u'Changed "900 Turbo 16" - a modifié : name (car [900 Turbo 16]) ; ')

    def test_logs_for_formsets(self):
        car = Car.objects.create(brand='Lada', name='Niva')

        CarOptionFormSet = inlineformset_factory(Car, CarOption, fields=['name'])
        formset = CarOptionFormSet({
            'caroption_set-INITIAL_FORMS': '0',
            'caroption_set-TOTAL_FORMS': '1',
            'caroption_set-MIN_NUM_FORMS': '0',
            'caroption_set-MAX_NUM_FORMS': '10',
            'caroption_set-0-id': '',
            'caroption_set-0-car': car.pk,
            'caroption_set-0-name': 'BVA',
            'caroption_set-0-DELETE': '',
        }, instance=car)
        formset.is_valid()
        formset.save()

        options.logs_for_formsets('change', self.request, car, [formset])
        log = LogEntry.objects.latest('pk')
        self.assertEqual(log.change_message, u'a modifié : name (car option [BVA]) ; ')



import csv


class UnicodeWriter(object):
    def __init__(self, f, **kwargs):
        self.writer = csv.writer(f, **kwargs)

    def writerow(self, row):
        self.writer.writerow(row)

    def writerows(self, rows):
        for row in rows:
            self.writerow(row)



from django.core.management.base import BaseCommand
from django.db.models import get_models, get_app
from django.contrib.auth.management import create_permissions

# http://djangosnippets.org/snippets/2398/

class Command(BaseCommand):
    args = '<app app ...>'
    help = 'reloads permissions for specified apps, or all apps if no args are specified'

    def handle(self, *args, **options):
        if not args:
            apps = []
            for model in get_models():
                apps.append(get_app(model._meta.app_label))
        else:
            apps = []
            for arg in args:
                apps.append(get_app(arg))
        for app in apps:
            create_permissions(app, get_models(), options.get('verbosity', 0))



import os
from django.core.files.storage import FileSystemStorage


class OverwriteStorage(FileSystemStorage):
    def _save(self, name, content):
        if self.exists(name):
            os.unlink(os.path.join(self.location, name))
        return super(OverwriteStorage, self)._save(name, content)

    def get_available_name(self, name, max_length=None):
        return name
